mod bind_dependencies;
mod call_descriptor;
mod enum_descriptor;
mod function_descriptor;
mod graph_builder_integration;
mod impl_from;
mod let_binding;
mod query_examples;
mod query_integration;
mod scope_insertion;
mod structure_descriptor;
mod variable_descriptor;
